@@ -30,9 +30,10 @@
 
 import json
 import unittest
+from binascii import unhexlify
 
 from Cryptodome.SelfTest.st_common import list_test_cases
-from Cryptodome.Util.py3compat import unhexlify, tobytes, bchr, b, _memoryview
+from Cryptodome.Util.py3compat import tobytes, bchr
 from Cryptodome.Cipher import AES
 from Cryptodome.Hash import SHAKE128
 
@@ -71,16 +72,16 @@ class SivTests(unittest.TestCase):
 
         cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
         ct2, tag2 = cipher.encrypt_and_digest(self.data_128)
-        self.assertEqual(ct1 + tag1, ct2 + tag2)
+        self.assertEquals(ct1 + tag1, ct2 + tag2)
 
     def test_nonce_must_be_bytes(self):
         self.assertRaises(TypeError, AES.new, self.key_256, AES.MODE_SIV,
-                          nonce='test12345678')
+                          nonce=u'test12345678')
 
     def test_nonce_length(self):
         # nonce can be of any length (but not empty)
         self.assertRaises(ValueError, AES.new, self.key_256, AES.MODE_SIV,
-                          nonce=b(""))
+                          nonce=b"")
 
         for x in range(1, 128):
             cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=bchr(1) * x)
@@ -95,7 +96,7 @@ class SivTests(unittest.TestCase):
         self.assertEqual(cipher.nonce, self.nonce_96)
 
         # By default, no nonce is randomly generated
-        self.assertFalse(hasattr(AES.new(self.key_256, AES.MODE_SIV), "nonce"))
+        self.failIf(hasattr(AES.new(self.key_256, AES.MODE_SIV), "nonce"))
 
     def test_unknown_parameters(self):
         self.assertRaises(TypeError, AES.new, self.key_256, AES.MODE_SIV,
@@ -120,11 +121,11 @@ class SivTests(unittest.TestCase):
 
     def test_data_must_be_bytes(self):
         cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
-        self.assertRaises(TypeError, cipher.encrypt, 'test1234567890-*')
+        self.assertRaises(TypeError, cipher.encrypt, u'test1234567890-*')
 
         cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
         self.assertRaises(TypeError, cipher.decrypt_and_verify,
-                          'test1234567890-*', b("xxxx"))
+                          u'test1234567890-*', b"xxxx")
 
     def test_mac_len(self):
         cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
@@ -239,10 +240,63 @@ class SivTests(unittest.TestCase):
         pt_test = cipher3.decrypt_and_verify(ct_ba, tag_ba)
 
         self.assertEqual(self.data_128, pt_test)
+    
+    def test_output_param(self):
 
-    import types
-    if _memoryview is type(None):
+        pt = b'5' * 16
+        cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
+        ct, tag = cipher.encrypt_and_digest(pt)
+
+        output = bytearray(16)
+        cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
+        res, tag_out = cipher.encrypt_and_digest(pt, output=output)
+        self.assertEqual(ct, output)
+        self.assertEqual(res, None)
+        self.assertEqual(tag, tag_out)
+        
+        cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
+        res = cipher.decrypt_and_verify(ct, tag, output=output)
+        self.assertEqual(pt, output)
+        self.assertEqual(res, None)
+
+    def test_output_param_memoryview(self):
+        
+        pt = b'5' * 16
+        cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
+        ct, tag = cipher.encrypt_and_digest(pt)
+
+        output = memoryview(bytearray(16))
+        cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
+        cipher.encrypt_and_digest(pt, output=output)
+        self.assertEqual(ct, output)
+        
+        cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
+        cipher.decrypt_and_verify(ct, tag, output=output)
+        self.assertEqual(pt, output)
+
+    def test_output_param_neg(self):
+
+        pt = b'5' * 16
+        cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
+        ct, tag = cipher.encrypt_and_digest(pt)
+
+        cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
+        self.assertRaises(TypeError, cipher.encrypt_and_digest, pt, output=b'0'*16)
+        
+        cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
+        self.assertRaises(TypeError, cipher.decrypt_and_verify, ct, tag, output=b'0'*16)
+
+        shorter_output = bytearray(15)
+        cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
+        self.assertRaises(ValueError, cipher.encrypt_and_digest, pt, output=shorter_output)
+        cipher = AES.new(self.key_256, AES.MODE_SIV, nonce=self.nonce_96)
+        self.assertRaises(ValueError, cipher.decrypt_and_verify, ct, tag, output=shorter_output)
+
+
+    import sys
+    if sys.version[:3] == "2.6":
         del test_memoryview
+        del test_output_param_memoryview
 
 
 class SivFSMTests(unittest.TestCase):
@@ -255,13 +309,13 @@ class SivFSMTests(unittest.TestCase):
         # Path INIT->ENCRYPT fails
         cipher = AES.new(self.key_256, AES.MODE_SIV,
                          nonce=self.nonce_96)
-        self.assertRaises(TypeError, cipher.encrypt, b("xxx"))
+        self.assertRaises(TypeError, cipher.encrypt, b"xxx")
 
     def test_invalid_init_decrypt(self):
         # Path INIT->DECRYPT fails
         cipher = AES.new(self.key_256, AES.MODE_SIV,
                          nonce=self.nonce_96)
-        self.assertRaises(TypeError, cipher.decrypt, b("xxx"))
+        self.assertRaises(TypeError, cipher.decrypt, b"xxx")
 
     def test_valid_init_update_digest_verify(self):
         # No plaintext, fixed authenticated data
@@ -330,6 +384,17 @@ class SivFSMTests(unittest.TestCase):
         self.assertRaises(TypeError, cipher.decrypt_and_verify, ct, tag)
 
 
+def transform(tv):
+    new_tv = [[unhexlify(x) for x in tv[0].split("-")]]
+    new_tv += [ unhexlify(x) for x in tv[1:5]]
+    if tv[5]:
+        nonce = unhexlify(tv[5])
+    else:
+        nonce = None
+    new_tv += [ nonce ]
+    return new_tv
+
+
 class TestVectors(unittest.TestCase):
     """Class exercising the SIV test vectors found in RFC5297"""
 
@@ -344,7 +409,7 @@ class TestVectors(unittest.TestCase):
     #
     #  A "Header" is a dash ('-') separated sequece of components.
     #
-    test_vectors = [
+    test_vectors_hex = [
       (
         '101112131415161718191a1b1c1d1e1f2021222324252627',
         '112233445566778899aabbccddee',
@@ -366,14 +431,7 @@ class TestVectors(unittest.TestCase):
       ),
     ]
 
-    for index, tv in enumerate(test_vectors):
-        test_vectors[index] = [[unhexlify(x) for x in tv[0].split("-")]]
-        test_vectors[index] += [unhexlify(x) for x in tv[1:5]]
-        if tv[5]:
-            nonce = unhexlify(tv[5])
-        else:
-            nonce = None
-        test_vectors[index].append(nonce)
+    test_vectors = [ transform(tv) for tv in test_vectors_hex ]
 
     def runTest(self):
         for assoc_data, pt, ct, mac, key, nonce in self.test_vectors:
@@ -398,6 +456,7 @@ class TestVectorsWycheproof(unittest.TestCase):
 
     def __init__(self):
         unittest.TestCase.__init__(self)
+        self._id = "None"
 
     def setUp(self):
         comps = "Cryptodome.SelfTest.Cipher.test_vectors.wycheproof".split(".")
@@ -450,6 +509,64 @@ class TestVectorsWycheproof(unittest.TestCase):
             self.test_decrypt(tv)
 
 
+class TestVectorsWycheproof2(unittest.TestCase):
+
+    def __init__(self):
+        unittest.TestCase.__init__(self)
+        self._id = "None"
+
+    def setUp(self):
+        comps = "Cryptodome.SelfTest.Cipher.test_vectors.wycheproof".split(".")
+        with open(pycryptodome_filename(comps, "aead_aes_siv_cmac_test.json"), "rt") as file_in:
+            tv_tree = json.load(file_in)
+
+        class TestVector(object):
+            pass
+        self.tv = []
+
+        for group in tv_tree['testGroups']:
+            for test in group['tests']:
+                tv = TestVector()
+
+                tv.id = test['tcId']
+                for attr in 'key', 'iv', 'aad', 'msg', 'ct', 'tag':
+                    setattr(tv, attr, unhexlify(test[attr]))
+                tv.valid = test['result'] != "invalid"
+                self.tv.append(tv)
+
+    def shortDescription(self):
+        return self._id
+
+    def test_encrypt(self, tv):
+        self._id = "Wycheproof Encrypt AEAD-AES-SIV Test #" + str(tv.id)
+
+        cipher = AES.new(tv.key, AES.MODE_SIV, nonce=tv.iv)
+        cipher.update(tv.aad)
+        ct, tag = cipher.encrypt_and_digest(tv.msg)
+        if tv.valid:
+            self.assertEqual(ct, tv.ct)
+            self.assertEqual(tag, tv.tag)
+
+    def test_decrypt(self, tv):
+        self._id = "Wycheproof Decrypt AEAD-AES-SIV Test #" + str(tv.id)
+
+        cipher = AES.new(tv.key, AES.MODE_SIV, nonce=tv.iv)
+        cipher.update(tv.aad)
+        try:
+            pt = cipher.decrypt_and_verify(tv.ct, tv.tag)
+        except ValueError:
+            assert not tv.valid
+        else:
+            assert tv.valid
+            self.assertEqual(pt, tv.msg)
+
+    def runTest(self):
+
+        for tv in self.tv:
+            self.test_encrypt(tv)
+            self.test_decrypt(tv)
+
+
 def get_tests(config={}):
     wycheproof_warnings = config.get('wycheproof_warnings')
 
@@ -458,6 +575,7 @@ def get_tests(config={}):
     tests += list_test_cases(SivFSMTests)
     tests += [ TestVectors() ]
     tests += [ TestVectorsWycheproof() ]
+    tests += [ TestVectorsWycheproof2() ]
     return tests
 
 
