@@ -28,7 +28,7 @@ import math
 import sys
 import struct
 from Cryptodome import Random
-from Cryptodome.Util.py3compat import _memoryview
+from Cryptodome.Util.py3compat import _memoryview, iter_range
 
 # Backward compatibility
 _fastmath = None
@@ -122,7 +122,7 @@ def GCD(x,y):
 def inverse(u, v):
     """The inverse of :data:`u` *mod* :data:`v`."""
 
-    u3, v3 = int(u), int(v)
+    u3, v3 = u, v
     u1, v1 = 1, 0
     while v3 > 0:
         q = u3 // v3
@@ -152,9 +152,9 @@ def getPrime(N, randfunc=None):
 def _rabinMillerTest(n, rounds, randfunc=None):
     """_rabinMillerTest(n:long, rounds:int, randfunc:callable):int
     Tests if n is prime.
-    Returns 0 when n is definitly composite.
+    Returns 0 when n is definitely composite.
     Returns 1 when n is probably prime.
-    Returns 2 when n is definitly prime.
+    Returns 2 when n is definitely prime.
 
     If randfunc is omitted, then Random.new().read is used.
 
@@ -175,7 +175,7 @@ def _rabinMillerTest(n, rounds, randfunc=None):
 
     tested = []
     # we need to do at most n-2 rounds.
-    for i in range (min (rounds, n-2)):
+    for i in iter_range (min (rounds, n-2)):
         # randomly choose a < n and make sure it hasn't been tested yet
         a = getRandomRange (2, n, randfunc)
         while a in tested:
@@ -186,7 +186,7 @@ def _rabinMillerTest(n, rounds, randfunc=None):
         if z == 1 or z == n_1:
             continue
         composite = 1
-        for r in range (b):
+        for r in iter_range(b):
             z = (z * z) % n
             if z == 1:
                 return 0
@@ -198,7 +198,7 @@ def _rabinMillerTest(n, rounds, randfunc=None):
     return 1
 
 def getStrongPrime(N, e=0, false_positive_prob=1e-6, randfunc=None):
-    """
+    r"""
     Return a random strong *N*-bit prime number.
     In this context, *p* is a strong prime if *p-1* and *p+1* have at
     least one large prime factor.
@@ -240,7 +240,7 @@ def getStrongPrime(N, e=0, false_positive_prob=1e-6, randfunc=None):
 
     # Use the accelerator if available
     if _fastmath is not None:
-        return _fastmath.getStrongPrime(int(N), int(e), false_positive_prob,
+        return _fastmath.getStrongPrime(long(N), long(e), false_positive_prob,
             randfunc)
 
     if (N < 512) or ((N % 128) != 0):
@@ -269,7 +269,7 @@ def getStrongPrime(N, e=0, false_positive_prob=1e-6, randfunc=None):
         # sieve the field
         for prime in sieve_base:
             offset = y % prime
-            for j in range ((prime - offset) % prime, len (field), prime):
+            for j in iter_range((prime - offset) % prime, len (field), prime):
                 field[j] = 1
 
         # look for suitable p[i] starting at y
@@ -352,7 +352,7 @@ def isPrime(N, false_positive_prob=1e-6, randfunc=None):
         randfunc = Random.get_random_bytes
 
     if _fastmath is not None:
-        return _fastmath.isPrime(int(N), false_positive_prob, randfunc)
+        return _fastmath.isPrime(long(N), false_positive_prob, randfunc)
 
     if N < 3 or N & 1 == 0:
         return N == 2
